/*
  This is the unique texture generator for the terrain
  
  Limitations: The base textures should be in 24bpp format.
  and the heightmap should be in 8bpp format 
  
  Added config file support
*/

#include <stdio.h>
#include "tga.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define CFG_FILE "texgen.cfg"

typedef struct 
{
  int hi, lo;
} limits;

int num_textures;
tga **pics, *hmap;
limits *ltable;

//For every texture image create a reference map and......
void parse_file();
void get_string(char *, int, FILE *);

void gen_tex_table(int num)
{
  pics=(tga **)malloc(num*sizeof(tga *));
  ltable=(limits *)malloc(num*sizeof(limits));
}

void add_to_tex_table(char *filename, int index, int hi, int lo)
{
  pics[index]=new tga(filename);
  ltable[index].lo=lo;
  ltable[index].hi=hi;
}

void destroy_tex_table()
{
  int i;
  for(i=0; i<num_textures; i++)
    {
      free(pics[i]);
    }
  free(pics);
  free(ltable);
}



void main(int argc, char **argv)
{
  tga *hmap;
  
  unsigned char *data;
  
  if (argc != 2)
    {
      printf("Usage: %s <8bppheightmap.tga>\n",argv[0]);
      exit(0);
    }
  
  
  parse_file();
  
  hmap=new tga(argv[1]);
  
  data=(unsigned char *)malloc(3*hmap->width*hmap->height);
  memset(data, 0, 3*hmap->width*hmap->height);
  
  //Main processing phase
  int offset=0;
  for(int y=0; y<hmap->height; y++)
    for(int x=0; x< hmap->width; x++)
      {
	unsigned char height=hmap->data[offset++];
	int clean=1;
	float alpha;
	
	for(int i=0; i<num_textures; i++)
	  {
	    //If within limits, add to data suitably
	    if ( (height >= ltable[i].lo) && (height <= ltable[i].hi))
	      {
		//Calculate alpha for the current texture
		alpha=(float)(height-ltable[i].lo) \
		  /(float)(ltable[i].hi-ltable[i].lo);
		
		// This is a real funky way of doing the alpha calculation but
		// who cares

		//Get a value betn 30 & 150
		float degrees=(alpha*120.0f)+30.0f; 
		alpha=sin(degrees*3.14159f/180.0f);
		//Just check the bounds
		if (alpha < 0.0f) alpha=0.0f;
		if (alpha > 1.0f) alpha=1.0f;
		
		colour24 col=pics[i]->getCol(x,y);

		int offset2=(offset<<1)+offset;
		if (clean) //write directly
		  {
		    data[offset2+1]=col.r;
		    data[offset2+2]=col.g;
		    data[offset2+3]=col.b;
		  }
		else
		  {
#define A1 alpha
#define A2 (1.0f-A1)

		    //Mix the two colours
		    data[offset2+1]=(A1*col.r+A2*data[offset2+1]);
		    data[offset2+2]=(A1*col.g+A2*data[offset2+2]);
		    data[offset2+3]=(A1*col.b+A2*data[offset2+3]);
		  }
		clean=0;
	      }
	  }
	}
  
  hmap->write_24bpp(hmap->width,hmap->height,data,"texture.tga");

  delete hmap;
  //This crashes in windows. I dunno why ?
  //free(data); 
  destroy_tex_table();
}


/*
  # SAMPLE CFG FILE   
  num_textures 2
  # filename1 range.....
  tex1.tga 0 127
  tex2.tga 127 255
*/
void parse_file()
{
  FILE *fp;
  char buffer[256];
  int num_read=0;
  char str[256];
  int hi,lo;
  
  if((fp=fopen(CFG_FILE,"r"))==NULL)
    {
      printf("Cannot open file %s\n", CFG_FILE);
      exit(0);
    }
  
  //read the first string
  get_string(buffer,256,fp);
  while(buffer[0] == '#' || buffer[0] == ' ' || buffer[0] == '\t')
    {
      get_string(buffer,256,fp);
    }
  //If not a comment
  //Get the number of textures
  sscanf(buffer,"num_textures %d",&num_textures);
  printf("TEXGEN: num_textures: %d\n", num_textures);
  gen_tex_table(num_textures);
  
  //Now that we got num_textures, we can proceed further
  while(num_read != num_textures)
    {
      get_string(buffer,256,fp);
      if(buffer[0] != '#')
	{
	  sscanf(buffer,"%s %d %d",str,&lo,&hi);
	  printf("TEXGEN: %s %5d %5d\n",str,lo,hi);
	  add_to_tex_table(str,num_read,hi,lo);
	  num_read++;
	}
    }
  fclose(fp);
}


/*
  Slow way to do it but I dont care
  Cut-pasted and modified from my CNC expt
*/
void get_string(char *buff, int size, FILE *stream)
{
  char ch;
  int i=0;
  while( (ch=(char)fgetc(stream)) != '\n')
    {
      if (i<size-1)
        {
          buff[i++]=ch;
        }
    }
  buff[i]='\0';
}





